package provide Calendar 0.1

#########################################################################
#                                                                       #
# Script:  calendar.tcl                                                 #
#                                                                       #
# Author:  Dave Freese, dave@darkmatterfactory.com                      #
#                                                                       #
# Provides:  calendar <widget path> [options]                           #
#            dateentry <widget path> [options]                          #
#                                                                       #
#    The command/widget supports the following options,                 #
#    (more to be added at later date).                                  #
#    -font           Used used for top labels.                          #
#    -datefont       Font used for number labels.                       #
#    -bg             Background for top labels.                         #
#    -fg             Foreground for top labels.                         #
#    -datebg         Background for number labels.                      #
#    -datefg         Foreground for number labels.                      #
#    -textvariable   Variable set when date is selected.                #
#    -highlightdate  Highlight the current date with                    #
#                    different font / background.                       #
#    -highlightfont  The font to use if highlighting the                #
#                    current date.                                      #
#    -highlightbg    The background color if highlighting               #
#                    the current date.                                  #
#    -highlightfg    The text color if highlighting the current date    #
#                                                                       #
#    Supported commands:                                                #
#                                                                       #
#    get        Return the last selected date.                          #
#    set        Set the month to show, give date in                     #
#               the format "1 month year".                              #
#    config     Configure the widget options.                           #
#    backward   Move the widget back a month.                           #
#    forward    Move the widget forward a month.                        #
#                                                                       #
# Purpose:  Provides a mega-widget allowing a particular date           #
#           to be chosen.                                               #
#                                                                       #
# Credits:                                                              #
#           Simon Edwards, (C) Proprius Consulting Ltd                  #
#           Bryan Oakley (oakley@channelpoint.com)                      #
#           Simon Edwards provided original "datechooser" widget upon   #
#           which the calendar widget is based.                         #
#           Bryan Oakley's combobox widget provides the framework       #
#           for creating megawidgets in a namespace!                    #
#                                                                       #
# Exit Codes:  0  Ok                                                    #
#                                                                       #
# Version:  v0.1.0                                                      #
# --------------------------------------------------------------------- #
#                                                                       #
#                                                                       #
# Version:  0.1.0                                                       #
# Date:    9 October, 2001                                              #
# Author:  Dave Freese, dave@darkmatterfactory.com                      #
# Change:  Original released version.                                   #
#                                                                       #
# Version:                                                              #
# Date:                                                                 #
# Author:                                                               #
# Change:                                                               #
#                                                                       #
#########################################################################

###############################################################################
#    calendar.tcl - tcl module to provide calendar mega-wdget                 #
#    Copyright (C) 2001 Dave Freese                                           #
#                                                                             #
#    This program is free software; you can redistribute it and/or modify     #
#    it under the terms of the GNU General Public License as published by     #
#    the Free Software Foundation; either version 2 of the License, or        #
#    (at your option) any later version.                                      #
#                                                                             #
#    This program is distributed in the hope that it will be useful,          #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of           #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            #
#    GNU General Public License for more details.                             #
#                                                                             #
#    You should have received a copy of the GNU General Public License        #
#    along with this program; if not, write to the Free Software              #
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA  #
#                                                                             #
###############################################################################

package require Tk 8.0
package require Megawidget 0.1

# Implements a calendar widget...

namespace eval ::calendar {
  namespace export calendar

  variable widgetOptions
  variable widgetCommands
}

proc ::calendar::calendar {w args} {
variable widgetOptions
variable widgetCommands

  # perform a one time initialization
    if {![info exists widgetOptions]} {
    Init
  }

  # We implement the new object as an object in its own
  # hidden namespace at present..

  eval Build $w $args

  return $w
}

proc ::calendar::DarkenColor {color val} {
variable cformat
  if {![info exists cformat]} {
    set red [lindex [winfo rgb . red] 0]
    switch -exact -- $red {
      15 { set cformat "#%01x%01x%01x" }
      255 { set cformat "#%02x%02x%02x" }
      4095 { set cformat "#%03x%03x%03x" }
      65535 { set cformat "#%04x%04x%04x" }
      default {set cformat "#%04x%04x%04x" }
    }
  }
  set rgb [winfo rgb . $color]
  return [format $cformat \
    [expr round ([lindex $rgb 0] * $val)] \
    [expr round ([lindex $rgb 1] * $val)] \
    [expr round ([lindex $rgb 2] * $val)]]
}

proc ::calendar::Init {} {
  variable widgetOptions
  variable widgetCommands
  variable mlist 
  variable mdays 
  variable namedays
  set mlist "January February March April May June July August September October November December"
  set mdays "31 28 31 30 31 30 31 31 30 31 30 31"
  set namedays "Sun Mon Tue Wed Thu Fri Sat"

  array set widgetOptions [list                    \
    -background     {background     Background}    \
    -bg             -background                    \
    -foreground     {foreground     Foreground}    \
    -fg             -foreground                    \
    -bd             -borderwidth                   \
    -font           {font           Font}          \
    -datefont       {datefont       DateFont}      \
    -dateformat     {dateformat     DateFormat}    \
    -datefg         {datefg         Datefg}        \
    -dfg            -datefg                        \
    -datebg         {datebg         Datebg}        \
    -dbg            -datebg                        \
    -textvariable   {textVariable   Variable}      \
    -highlightdate  {highlightdate  HighlightDate} \
    -highlightfont  {highlightfont  HighlightFont} \
    -highlightbg    {highlightbg    HighlightBg}   \
    -highlightfg    {highlightfg    HighlightFg}   \
    -cdate          {cdate          Cdate}         \
    -width          {width          Width}         \
    -height         {height         Height}
  ]

  button .dummybutton
  label  .dummylabel
    set btnbg [.dummybutton cget -background]
    set btnfg [.dummybutton cget -foreground]
    set lblbg [.dummylabel  cget -background]
    set lblfg [.dummylabel  cget -foreground]
  destroy .dummybutton
  destroy .dummylabel

  option add *Calendar.foreground     $lblfg
  option add *Calendar.background     $lblbg
  option add *Calendar.datefont       fixed
  option add *Calendar.highlightfont  fixed
  option add *Calendar.highlightdate  1
  option add *Calendar.datefg         $btnfg
  option add *Calendar.datebg         $btnbg
  option add *Calendar.highlightfg    $btnfg
  option add *Calendar.highlightbg    [DarkenColor $btnbg 0.8]
  option add *Calendar.dateformat     "dd mmm yyyy"
  option add *Calendar.highlightdate) 1
  option add *Calendar.cdate)         ""
# following dimensions are hardcoded, they were obtained using
# [winfo reqwidth objectname] [winfo reqheight objectname] 
# after a calendar object was created and mapped to the display
# If anyone knows how to obtain the physical size of an object without
# it being mapped to the display please let me know.
# You may have to change the width & height (pixels) to reflect your
# display characteristics/
  option add *Calendar.width          172
  option add *Calendar.height         120

  if {$::tcl_platform(platform) == "unix"} {
    option add *Calendar.datefont \
      [font create -slant roman -weight normal -family Courier -size 10]
    option add *Calendar.highlightfont \
      [font create -slant roman -weight bold -family Courier -size 10]
  } else {
    option add *Calendar.datefont \
      [font create -slant roman -weight normal -family Courier -size 8]
    option add *Calendar.highlightfont \
      [font create -slant roman -weight bold -family Courier -size 8]
  }


  set widgetCommands [list  backward configure cget forward get set ]

  image create bitmap ::calendar::bmpPrevYr -data {
    #define PrevYr_width 10
    #define PrevYr_height 10
    static unsigned char PrevYr_bits[] = {
      0x10,0x02,0x18,0x03,0x9c,0x03,0xde,0x03,0xff,0x03,
      0xff,0x03,0xde,0x03,0x9c,0x03,0x18,0x03,0x10,0x02 };
  }
  image create bitmap ::calendar::bmpPrevMn -data {
    #define test.bmp_width 10
    #define test.bmp_height 10
    static unsigned char test.bmp_bits[] = {
      0x80,0x00,0xc0,0x00,0xe0,0x00,0xf0,0x00,0xf8,0x00,
      0xf8,0x00,0xf0,0x00,0xe0,0x00,0xc0,0x00,0x80,0x00};
  }
  image create bitmap ::calendar::bmpNextMn -data {
    #define test2_width 10
    #define test2_height 10
    static unsigned char test2_bits[] = {
      0x04,0x00,0x0c,0x00,0x1c,0x00,0x3c,0x00,0x7c,0x00,
      0x7c,0x00,0x3c,0x00,0x1c,0x00,0x0c,0x00,0x04,0x00};
  }
  
  image create bitmap ::calendar::bmpNextYr -data {
    #define NextYr_width 10
    #define NextYr_height 10
    static unsigned char NextYr_bits[] = {
      0x21,0x00,0x63,0x00,0xe7,0x00,0xef,0x01,0xff,0x03,
      0xff,0x03,0xef,0x01,0xe7,0x00,0x63,0x00,0x21,0x00};
  }
}

proc ::calendar::Build {w args} {
variable widgetOptions
variable widgetCommands
variable building 1
variable namedays

  if {[winfo exists $w]} {
    error "window name \"$w\" already exists"
  }

  namespace eval ::calendar::$w {

    variable ctime  [clock seconds]
    variable cmonth [clock format $ctime -format "%B"]
    variable cyear  [clock format $ctime -format "%Y"]

    variable widgets
    variable options

    set widgets(boo) boo
    unset widgets(boo)

    set options(boo) boo
    unset options(boo)
  }

  upvar ::calendar::${w}::widgets widgets
  upvar ::calendar::${w}::options options
  upvar ::calendar::${w}::cmonth  cmonth
  upvar ::calendar::${w}::cyear   cyear

# Create a named global variable for use by "get"
  global ::calendar::${w}::tempvar
  
# Create the popup frame and its subordinates
  set widgets(this)   [frame $w \
                        -class Calendar \
                        -takefocus 0 ] 
  set widgets(f1)     [frame $w.f1 \
                        -relief groove \
                        -borderwidth 2 \
                        -takefocus 0]

  foreach name [array names widgetOptions] {
    if {[llength $widgetOptions($name)] == 1} continue
    set optName  [lindex $widgetOptions($name) 0]
    set optClass [lindex $widgetOptions($name) 1]
    set value [option get $w $optName $optClass]
    set options($name) $value
  }

# create a hidden entry widget to act as -textvariable transfer agent
# back to calling proc
  set widgets(-hidden) [entry $widgets(this).hidden]
  

# Retrieve options from .Xdefaults file & set the values,

  foreach name [array names widgetOptions] {
    if {[llength $widgetOptions($name)] == 1} continue
    set optName  [lindex $widgetOptions($name) 0]
    set optClass [lindex $widgetOptions($name) 1]
    set value [option get $w $optName $optClass]
    if {[string length $value] != 0} {
      set options($name) $value
    }
  }

# Now take care of configuration items passed on the arguement list...
#puts "$widgets(this) $args"
  if { [llength $args] > 0 } {
    if { [catch "::calendar::Configure $widgets(this) $args" error] } {
      catch {destroy $w}
      error $error
    }
  }

# If a date was not specified, then highlight todays date

  if { $options(-cdate) == ""} {
    set options(-cdate) [clock format [clock seconds] -format "%d %B %Y"]
    if { [string index $options(-cdate) 0] == "0" } {
      set options(-cdate) [string range $options(-cdate) 1 end]
    }
  }
    
# Create the Next/Prev Year & Next/Prev Month buttons 
  set widgets(prvYr) [button $w.f1.prevYr -image ::calendar::bmpPrevYr \
          -takefocus 0 \
          -padx 0 -pady 0 -borderwidth 1 \
          -command "::calendar::prev_year $w"]
  set widgets(prev)  [button $w.f1.prev -image ::calendar::bmpPrevMn \
          -takefocus 0 \
          -padx 0 -pady 0 -borderwidth 1 \
          -command "::calendar::prev_month $w"]
  set widgets(next)  [button $w.f1.next -image ::calendar::bmpNextMn \
          -takefocus 0 \
          -padx 0 -pady 0 -borderwidth 1 \
          -command "::calendar::next_month $w"]
  set widgets(nxtYr) [button $w.f1.nextYr -image ::calendar::bmpNextYr \
          -takefocus 0 \
          -padx 0 -pady 0 -borderwidth 1 \
          -command "::calendar::next_year $w"]

# Create label for MMM YYYY title  
  set widgets(label)  [label $w.f1.lab \
                         -width 10 \
                         -font $options(-highlightfont) \
                         -text "$cmonth $cyear" \
                         -takefocus 0]

  grid $widgets(prvYr) -row 0 -column 0
  grid $widgets(prev)  -row 0 -column 1
  grid $widgets(label) -row 0 -column 2 -columnspan 3
  grid $widgets(next)  -row 0 -column 5
  grid $widgets(nxtYr) -row 0 -column 6

# Create the buttons for the calendar array
  for {set x 0} {$x < 7} {incr x} {
    for {set y 1} {$y < 8} {incr y} {
      if { $y == 1 } {
        label $widgets(f1).b${y}_$x \
          -text [lindex $namedays $x] \
          -font $options(-datefont)
      } else {
        button $widgets(f1).b${y}_$x \
          -width 1 \
          -relief groove \
          -padx 0 -pady 0 -borderwidth 0 \
          -font $options(-datefont)
      }
      grid $widgets(f1).b${y}_$x -in $widgets(f1) -row $y -column $x \
         -sticky nsew
    }
  }

# Fill in current buttons...

  ::calendar::update_buts $w

# Store the proc to control the widget

  set widgets(frame) ::calendar::${w}::${w}

  # Pack the widgets...

  pack $widgets(f1)    -side bottom -fill both -expand yes

  if {[info exists options(-value)] \
    && [string length $options(-value)] == 0} {
    unset options(-value)
  }

  rename ::$w $widgets(frame)

  # Set the class bindings for the widget
  bind calendar <Destroy> [list ::megawidget::DestroyHandler %W]

  # Now create the proc to handle the widget...

  proc ::$w {command args} \
    "eval ::calendar::WidgetProc $w \$command \$args"

  set building 0
  
  return ""
}

proc ::calendar::Configure {w args} {
  variable widgetOptions
  variable building
  upvar ::calendar::${w}::widgets widgets
  upvar ::calendar::${w}::options options
  upvar ::calendar::${w}::cmonth cmonth
  upvar ::calendar::${w}::cyear cyear

  # Return all results to the user
  if {[llength $args] == 0} {
    set results {}
    foreach opt [lsort [array names widgetOptions]] {
      if {[llength $widgetOptions($opt)] == 1} {
        set alias $widgetOptions($opt)
        set optName $widgetOptions($alias)
        lappend results [list $opt $optName]
      } else {
        set optName  [lindex $widgetOptions($opt) 0]
        set optClass [lindex $widgetOptions($opt) 1]
        set default {}
        set default [option get $w $optName $optClass]
        set setopt {}
        if {[info exists options($opt)]} {set setopt $options($opt)}
        lappend results [list $opt $optName $optClass $default $setopt]
      }
    }
    return $results
  }

  # Are we looking for config info for an option?
  if {[llength $args] == 1} {
    set opt [::megawidget::Canonize $w option $args]
    set optName  [lindex $widgetOptions($opt) 0]
    set optClass [lindex $widgetOptions($opt) 1]
    set default [option get $w $optName $optClass]
    set setopt {}
    if {[info exists options($opt)]} {set setopt $options($opt)}
    set results [list $opt $optName $optClass \
      $default $setopt]
    return $results
  }

  # if we have an odd number of values, bail.
  if {[expr {[llength $args]%2}] == 1} {
    # hmmm. An odd number of elements in args
    error "value for \"[lindex $args end]\" missing"
  }

  # deal with arg/value pairs, putting them in an array...
  foreach {name value} $args {
    set name [::megawidget::Canonize $w option $name]
    set opts($name) $value
  }

  foreach option [array names opts] {
    set newValue $opts($option)
    if {[info exists options($option)]} {
      set oldValue $options($option)
    }

    switch -- $option {
      -background {
        if { "$newValue" != "" } {
          $widgets(frame) configure -background $newValue
          $widgets(f1)    configure -background $newValue
          $widgets(prev)  configure -background $newValue
          $widgets(next)  configure -background $newValue
          $widgets(prvYr) configure -background $newValue
          $widgets(nxtYr) configure -background $newValue
          $widgets(label) configure -background $newValue
        }
        set options($option) $newValue
      }
      -foreground {
        if { "$newValue" != "" } {
          $widgets(prev)  configure -foreground $newValue
          $widgets(next)  configure -foreground $newValue
          $widgets(prvYr) configure -foreground $newValue
          $widgets(nxtYr) configure -foreground $newValue
          $widgets(label)  configure -foreground $newValue
          set options($option) $newValue
        }
      }
    -dateformat {
      if {"$newValue" != "" } {
        set options($option) $newValue
      }
    }
      -datebg {
        if { "$newValue" != "" } {
          $widgets(f1) configure -background $newValue
          for {set x 0} {$x < 7} {incr x} {
            $widgets(f1) config -bg $newValue
            for {set y 2} {$y < 8} {incr y} {
              $widgets(f1).b${y}_$x config -bg $newValue
            }
          }
        }
        set options($option) $newValue
      }
      -datefg {
        if { "$newValue" != "" } {
          for {set x 0} {$x < 7} {incr x} {
            $widgets(f1) config -fg $newValue
            for {set y 2} {$y < 8} {incr y} {
              $widgets(f1).b${y}_$x config -fg $newValue
            }
          }
        }
        set options($option) $newValue
      }
      -textvariable {
        set options($option) $newValue
        $widgets(-hidden) configure -textvariable $newValue
      }
      -cdate {
#check for a valid date 
        if {[catch {clock scan $newValue} result]} {
          return
        }
        set tempdate [clock format [clock scan $newValue] -format "%d %B %Y"]
        if { [string index $tempdate 0] == "0" } {
          set tempdate [string range $tempdate 1 end]
        }
        set options($option) $tempdate
        set cmonth [lindex $tempdate 1]
        set cyear [lindex $tempdate 2]
        if {$building == 0} {::calendar::update_buts ${w}}
      }
      -highlightdate {
        if { $newValue == "" } {
          set newValue 0
        }
        if { ! [info exists options(-datefont)] } {
          set options(-datefont) \
          [font create -slant roman -weight normal \
                     -family Helvetica -size 10]
        }
        for {set x 0} {$x < 7} {incr x} {
          for {set y 2} {$y < 8} {incr y} {
            set _cdate [lindex [$widgets(f1).b${y}_$x config -text] 4]
            set _cdate "$_cdate $cmonth $cyear"
            if { "$_cdate" == "$options(-cdate)" && $newValue == 1} {
              $widgets(f1).b${y}_$x config \
              -font $options(-highlightfont) \
              -bg $options(-highlightbg) \
              -fg $options(-highlightfg)
            } else {
              $widgets(f1).b${y}_$x config \
              -font $options(-datefont) \
              -bg $options(-datebg) \
              -fg $options(-datefg)            }
          }
        }
        set options($option) $newValue
      }
      -font {
        if { $newValue == "" } {
          set newValue [font create -slant roman -weight bold \
                                -family Courier -size 8]
        }
        $widgets(label) configure -font $newValue
        set options($option) $newValue
      }
      -datefont {
        if { $newValue == "" } {
          set newValue [font create -slant roman -weight normal \
                                -family Courier -size 8]
        }
        for {set x 0} {$x < 7} {incr x} {
          for {set y 2} {$y < 8} {incr y} {
            $widgets(f1).b${y}_$x config -font $newValue
          }
        }
        set options($option) $newValue
      }
      -highlightfont {
        if { $newValue == "" } {
          set newValue [font create -slant roman -weight bold \
                                -family Courier -size 8]
        }
        if { $options(-highlightdate) == 1 } {
          for {set x 0} {$x < 7} {incr x} {
            for {set y 2} {$y < 8} {incr y} {
              set _cdate [lindex [$widgets(f1).b${y}_$x config -text] 4]
              set _cdate "$_cdate $cmonth $cyear"
              if { "$_cdate" == "$options(-cdate)" } {
                $widgets(f1).b${y}_$x config -font $newValue
              }
            }
          }
        }
        set options($option) $newValue
      }
      -highlightbg {
        if { $newValue == "" } {
           set newValue systemHighlight
        }
        for {set x 0} {$x < 7} {incr x} {
          for {set y 2} {$y < 8} {incr y} {
            set _cdate [lindex [$widgets(f1).b${y}_$x config -text] 4]
            set _cdate "$_cdate $cmonth $cyear"
            if { "$_cdate" == "$options(-cdate)" && 
                 $options(-highlightdate) == 1} {
              $widgets(f1).b${y}_$x config -font $options(-highlightfont) \
              -bg $newValue
            } else {
              $widgets(f1).b${y}_$x config -font $options(-datefont) \
              -bg $options(-datebg) \
              -fg $options(-datefg)
            }
          }
        }
        set options($option) $newValue
      }
    }
  }
}

# This is the actual command that gets called when the user attempts
# to use the widget object name as a procedure.

proc ::calendar::WidgetProc {w command args} {
upvar ::calendar::${w}::widgets widgets
upvar ::calendar::${w}::options options
upvar ::calendar::${w}::cmonth cmonth
upvar ::calendar::${w}::cyear cyear
global ::calendar::${w}::tempvar

  set command [::megawidget::Canonize $w command $command]

  set result ""

  switch $command {
    cget {
      if {[llength $args] != 1} {
        error "wrong # args: should be $w cget option"
      }
      set opt [::megawidget::Canonize $w option [lindex $args 0]]
      if {$opt == "-value"} {
        set result [$widget(entry) get]
      } else {
        set result $options($opt)
      }
    }
    
    configure {
      set result [eval ::calendar::Configure {$w} $args]
    }
    set {
      # Set the date to look at ...
      set xx [clock scan "[lindex $args 0]"]
      if { [catch {set xx [clock scan "[lindex $args 0]"]}] } {
        error "Invalid date specified for \"set\" command."
      } else {
        set cmonth [clock format $xx -format "%B"]
        set cyear [clock format $xx -format "%Y"]
        # Now change the dates shown...
        update_buts $w
      }
    }
    backward {
      ::calendar::prev_month $w
    }
    forward {
      ::calendar::next_month $w
    }
    backyear {
      ::calendar::prev_year $w
    }
    forwardyear {
      ::calendar::next_year $w
    }
    get {
      tkwait variable ::calendar::${w}::tempvar
      return [set ::calendar::${w}::tempvar]
    }
    default {
      error "bad option \"$command\""
    }
  }
}

# This is called when the month changes, or the widget is first
# created...

proc ::calendar::update_buts {w} {
upvar ::calendar::${w}::widgets widgets
upvar ::calendar::${w}::cmonth cmonth
upvar ::calendar::${w}::cyear cyear
upvar ::calendar::${w}::options options
variable mlist 
variable mdays 
variable namedays

# disable the date buttons in the first and last weeks...
  foreach x {0 1 2 3 4 5 6} {
    foreach y {2 6 7} { 
      $widgets(f1).b${y}_${x} config \
        -text "" \
        -state disabled \
        -relief flat \
        -bg $options(-datebg) \
        -fg $options(-datefg)
    }
  }

  set cmon [lsearch $mlist $cmonth]
  set maxdays [lindex $mdays $cmon]
  if {$cmon == 1} {
    if {[expr $cyear % 400] == 0} {
      incr maxdays
    } else {
      if {[expr $cyear % 100] != 0 && [expr $cyear % 4] == 0} {
        incr maxdays
      }
    }
  }

  set osecs [clock scan "1 $cmonth $cyear"]
  $widgets(label) config -text "[clock format $osecs -format "%b %y"]"

  set x     [lsearch -exact $namedays [clock format $osecs -format "%a"]]
  set y     2
  set dom   1
  incr cmon
  
  while {$dom <= $maxdays} {
    $widgets(f1).b${y}_$x config \
      -text "$dom"  \
      -state normal \
      -command "::calendar::but_hit $w $cmon $dom $cyear" \
      -font $options(-datefont) \
      -bg $options(-datebg) \
      -fg $options(-datefg)
  
    set _cdate "$dom $cmonth $cyear"
    if { "$_cdate" == "$options(-cdate)" && $options(-highlightdate) == 1 } {
      $widgets(f1).b${y}_$x config \
        -font $options(-highlightfont) \
        -bg $options(-highlightbg) \
        -fg $options(-highlightfg)
    }
    incr dom
    incr x
    if { $x == 7 } { incr y; set x 0 }
  }
}


# Called to update textvariable values as required...

proc ::calendar::but_hit {w m d y} {
  upvar ::calendar::${w}::options options
  upvar ::calendar::${w}::widgets widgets
  global ::calendar::${w}::tempvar

  variable mlist

  switch -exact -- $options(-dateformat) {
    "mm/dd/yy"    { 
      if { $y < 2000 } {
        incr y -1900
      } else {
        incr y -2000
      } 
      set datestr [format "%02d/%02d/%02d" $m $d $y] 
    }
    "mm/dd/yyyy"  { 
      set datestr [format "%02d/%02d/%4d" $m $d $y] 
    }
    "dd mmm yy"   {
      if { $y < 2000 } {
        incr y -1900
      } else {
        incr y -2000
      } 
      set datestr [format "%d %s %02d" $d [lindex $mlist [incr m -1]] $y] 
    }
    "dd mmm yyyy" { 
      set datestr [format "%d %s %d" $d [lindex $mlist [incr m -1]] $y] 
    }
    default { 
      set datestr [format "%d %s %d" $d [lindex $mlist [incr m -1]] $y] 
    }
  }
  set ::calendar::${w}::tempvar $datestr
# Pass the selected date (in string format) to the calling procedure
# via the -textvariable attribute of the hidden entry widget
  $widgets(-hidden) delete 0 end
  $widgets(-hidden) insert 0 $datestr

}

proc ::calendar::prev_month w {
  upvar ::calendar::${w}::cmonth cmonth
  upvar ::calendar::${w}::cyear cyear
  variable mlist

  set cmon [lsearch -exact $mlist $cmonth]
  if { $cmon == 0 } {
    set cmon 11
    incr cyear -1
  } else {
    incr cmon -1
  }
  set cmonth [lindex $mlist $cmon]
  update_buts $w
}

proc ::calendar::next_month w {
upvar ::calendar::${w}::cmonth cmonth
upvar ::calendar::${w}::cyear cyear
variable mlist

  set cmon [lsearch -exact $mlist $cmonth]
  if { $cmon == 11 } {
    set cmon 0
    incr cyear
  } else {
    incr cmon
  }
  set cmonth [lindex $mlist $cmon]
  update_buts $w
}

proc ::calendar::prev_year w {
upvar ::calendar::${w}::cyear cyear
  incr cyear -1
  update_buts $w
}

proc ::calendar::next_year w {
upvar ::calendar::${w}::cyear cyear
  incr cyear
  update_buts $w
}



